`timescale 1 ns/1 ns

localparam  CLK_MHZ    = 100          ; // Frequency of simulated clock
localparam  CLK_PERIOD = 1000/CLK_MHZ ; // Period of simulated clock.
localparam  CMD_COUNT  = 10           ; // Number of commands to send to PSG.
localparam  ENDTIME    = 1000 * 1000  ; // Number of ns to stop simulation at.

module jt49_tb ();

    // Test parameters
    parameter       DIVISOR  = 28'd56  ; // clock divisor value
    parameter       CMD_CLKS = DIVISOR ; // numbers of clk per command.
    parameter [1:0] COMP     = 2'b00   ;

    // PSG command sequencer
    reg [ 3:0] step = 0 ; // instruction step
    reg        bc   [0:CMD_COUNT-1] = '{  0,  1,  0,  1,  0,  1,  0,  1,  0,  0 } ;
    reg        bdir [0:CMD_COUNT-1] = '{  0,  1,  1,  1,  1,  1,  1,  1,  1,  0 } ;
    reg [ 7:0] data [0:CMD_COUNT-1] = '{  0,  0, 37,  1,  1,  8, 14,  7,  7,  0 } ;
    wire[ 1:0] ctrl = '{ bdir[step], bc[step] } ;

    // Signal declarations
    reg        clk        = 0 ;
    reg [23:0] f_count    = 0 ;
    wire       p_div          ;
    wire       p_stb          ;
    reg        reset      = 0 ;
    reg        wr_n, cs_n     ;
    reg [ 3:0] addr           ;
    reg        addr_ok        ;
    reg [ 7:0] din_latch      ;

    // Instantiate DUT
    clock_strobe #(

        .DIVISOR ( DIVISOR )

    ) DUT (

        .clk_i   ( clk     ), // source clock for strobe (divided by DIVISOR if >1)
        .clk_o   ( p_div   ), // divided clock 50:50 duty cycle output
        .strb_o  ( p_stb   )  // divided clock strobe output

    );

    // Instantiate PSG
    jt49 #(

        .COMP     ( COMP      )

    ) PSG (

        .rst_n    ( reset     ),
        .clk      ( clk       ),
        .clk_en   ( p_stb     ),
        .addr     ( addr[3:0] ),
        .cs_n     ( cs_n      ),
        .wr_n     ( wr_n      ),
        .din      ( din_latch ),
        .sel      ( 1'b1      ),
        .dout     ( dout      ),
        .sound    ( sound     ),
        .A        (           ),
        .B        (           ),
        .C        (           ),
        .sample   (           ),
        .IOA_in   (           ),
        .IOA_out  (           ),
        .IOB_in   (           ),
        .IOB_out  (           )

    );
    
    initial begin

        clk     = 1'b0  ;
        f_count = 24'b0 ;
        step    = 3'b0  ;

        //#(CLK_PERIOD*5) reset = 1'b1 ; // release reset after 5 clocks

        #(CLK_PERIOD*CMD_CLKS)  // Wait for CMD_CLKS oscillations of the system clk period before releasing reset

        reset   = 1'b1  ;       // release reset.

        #(CLK_PERIOD*CMD_CLKS)  // Wait for CMD_CLKS oscillations of the system clk period before sending commands.

        // Count the command counter until the end, waiting a CMD_CLKS between each command step.
        for (step = 0; step < (CMD_COUNT - 1); step++ ) #(CLK_PERIOD*CMD_CLKS) ;

    end

    // Generate simulated clock
    always begin

		clk = 1'b1 ;
		#(CLK_PERIOD/2) ;     // high for 1*timescale

		clk = 1'b0 ;
		#(CLK_PERIOD/2) ;	  // low for 1*timescale
        
	end

    // convert bc/bdir controls to jt49 interface
    always @(posedge clk) begin

        if( !reset ) begin
            wr_n    <= 1'b1;
            cs_n    <= 1'b1;
            addr    <= 4'd0;
            addr_ok <= 1'b1;
        end else begin
            case ( ctrl )
                2'b00: begin
                    wr_n      <= 1'b1 ;
                    cs_n      <= 1'b1 ;
                end
                2'b01: begin
                    //{ wr_n, cs_n } <= addr_ok ? 2'b10 : 2'b11 ;
                    wr_n      <= 1'b1     ;
                    cs_n      <= !addr_ok ;
                end
                2'b10: begin
                    //{ wr_n, cs_n } <= addr_ok ? 2'b00 : 2'b11 ;
                    wr_n      <= !addr_ok ;
                    cs_n      <= !addr_ok ;
                    din_latch <= data[step] ;
                end
                2'b11: begin
                    //{ wr_n, cs_n } <= 2'b11 ;
                    wr_n      <= 1'b1 ;
                    cs_n      <= 1'b1 ;
                    addr      <= data[step][3:0] ;
                    addr_ok   <= data[step][7:4] == 4'd0 ;
                end
            endcase
        end

    end

    always #(ENDTIME) $stop ; // Stop simulation from going on forever

    always @(posedge clk) if (p_stb) f_count <= f_count + 1 ;

endmodule
